# Copyright 2009 Simon Schampijer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Zim Activity: A wiki activity."""

import sys, os
from gi.repository import Gtk
import logging

from gettext import gettext as _

from sugar3.activity import activity
from sugar3.graphics.toolbarbox import ToolbarBox
from sugar3.activity.widgets import ActivityToolbarButton
from sugar3.activity.widgets import TitleEntry
from sugar3.activity.widgets import StopButton
from sugar3.activity.widgets import ShareButton
from sugar3.activity.widgets import DescriptionItem
from sugar3.graphics.toolbutton import ToolButton
from sugar3.graphics.radiotoolbutton import RadioToolButton
from sugar3.graphics.toggletoolbutton import ToggleToolButton
from sugar3.graphics import style
from sugar3.datastore import datastore

from toolbar_utils import (radio_factory, button_factory, separator_factory,
                           combo_factory, label_factory)
from subprocess import call, Popen, PIPE

from urllib2 import urlopen
from path import path
import datetime
from sftp import sftp
xo1 = False
try:
    import Image
except:
    xo1 = True

try:
    import gconf
except:
    using_gconf = False
else:
    using_gconf = True

MFG_DATA_PATHS = ['/ofw/mfg-data/', '/proc/device-tree/mfg-data/']
WORKPATH = path('/home/olpc/Documents')

#constants
imposition = [5,2,7,0,3,4,1,6]
qsize = (700,450)
a4size = (2800,1800)

class ZimActivity(activity.Activity):
    """ZimActivity class as specified in activity.info"""

    def __init__(self, handle):
        """Set up the Zim activity."""
        activity.Activity.__init__(self, handle)
        self._file_path = self._jobject.file_path
        if self._file_path:
            print 'file_path', self._file_path
            #unzip zipped file to /home/olpc/Documents/namebase
            cmd = 'unzip ' +  self._file_path +' -d /home/olpc/Documents/'
            print cmd
            call(cmd,shell=True)
        else:
            print 'file_path', 'None'

        # we do not have collaboration features
        # make the share option insensitive
        self.max_participants = 1

        # toolbar with the new toolbar redesign
        toolbar_box = ToolbarBox()

        # Activity toolbar
        activity_button_toolbar = ActivityToolbarButton(self)
        toolbar_box.toolbar.insert(activity_button_toolbar, 0)
        activity_button_toolbar.show()

        self._add_zim_controls(toolbar_box.toolbar)

        #title_entry = TitleEntry(self)
        #toolbar_box.toolbar.insert(title_entry, -1)
        #title_entry.show()

        #description_item = DescriptionItem(self)
        #toolbar_box.toolbar.insert(description_item, -1)
        #description_item.show()

        #share_button = ShareButton(self)
        #toolbar_box.toolbar.insert(share_button, -1)
        #share_button.show()
        
        separator = Gtk.SeparatorToolItem()
        separator.props.draw = False
        separator.set_expand(True)
        toolbar_box.toolbar.insert(separator, -1)

        toolbar_box.toolbar.insert(StopButton(self), -1)

        #stop_button = StopButton(self)
        #stop_button.props.accelerator = '<Ctrl>q'
        #toolbar_box.toolbar.insert(stop_button, -1)
        #stop_button.show()

        self.set_toolbar_box(toolbar_box)
        toolbar_box.show_all()

        # on resume, unzip file to /home/olpc/Documents
        if self._file_path:
            cmd = 'unzip -d /home/olpc/Documents ' + self._file_path
            print 'cmd', cmd
            pid = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
            result, err = pid.communicate()
            if result:
                print 'result', result
            if err:
                print 'err', err

        # set up screen
        self.screen = Gtk.VBox()
        self.fixed = Gtk.Fixed()
        # label with the text, make the string translatable
        self.label = Gtk.Label(_("Zim!"))
        self.label.show()
        self.fixed.put(self.label, 200,200)
        self.screen.pack_start(self.fixed, False, False, 0)

        self.set_canvas(self.screen)
        self.screen.show_all()

        _metadata = {}
        self._metadata = _metadata

    def _add_zim_controls(self, display_toolbar):

        # Four buttons: zim, print booklet, upload, inspect 
        zim_button = ToolButton()
        zim_button.set_tooltip(_('Zim'))
        zim_button.connect("clicked", self._launch_zim_cb)
        display_toolbar.insert(zim_button, -1)
        zim_button.show()

        if not xo1:
            print_button = ToolButton()
            print_button.set_tooltip(_('Booklet'))
            print_button.connect("clicked", self._print_booklet_cb)
            display_toolbar.insert(print_button, -1)
            print_button.show()

        upload_button = ToolButton()
        upload_button.set_tooltip(_('Upload'))
        upload_button.connect("clicked", self._upload_cb)
        display_toolbar.insert(upload_button, -1)
        upload_button.show()

        inspect_button = ToolButton()
        inspect_button.set_tooltip(_('Inspect'))
        inspect_button.connect("clicked", self._inspect_cb)
        display_toolbar.insert(inspect_button, -1)
        upload_button.show()

        # upload folder
        folder_item = Gtk.ToolItem()
        folder = Gtk.Entry()
       	folder.set_text('untitled')
        #folder.connect('focus_out_event',self._upload_cb)
        folder.set_alignment(0)
        folder.set_width_chars(20)
        folder_item.add(folder)
        display_toolbar.insert(folder_item, -1)
        folder_item.show()
        self.folder = folder

    def _launch_zim_cb(self, button):
        # launch Zim
        pid = Popen('zim',shell=True)
        pid.communicate()
        #print 'Zim terminated'
        #self.close(True)
        #sys.exit(0)

    def _print_booklet_cb(self, widget):
        #this procedure assumes set of pages name-1.png, name-2.png, etc.
        #actually we will have folder with file per page (sub to main file)
        print 'print_booklet_cb'
        name = self.folder.get_text()
        print 'booklet title', name
        #if name.find(' ') < 0:
            #return
        name = name.strip()
        all_pages = path('/home/olpc/Documents').files(name+'*.png')
        all_pages.sort()
        print len(all_pages)
        #find cover pages
        cover_pages = []
        pages = []
        for page in all_pages:
            if '-c' in page.namebase:
               cover_pages.append[page]
            else:
               pages.append(page)
        pg1, pg2 = page_format(pages, imposition)
        cover_pg1, cover_pg2 = page_format(cover_pages,cover_imposition)

        #save in /home/olpc/Documents
        pag1 = pg1.crop((0,0,2*qx,2*qy))
        pag2 = pg2.crop((0,0,2*qx,2*qy))
        pag1.save('/home/olpc/Documents/'+name+'-page1.png')
        pag2.save('/home/olpc/Documents/'+name+'-page2.png')
        #save in /home/olpc/Documents
        pag1 = cover_pg1.crop((0,0,2*qx,2*qy))
        pag2 = cover_pg2.crop((0,0,2*qx,2*qy))
        pag1.save('/home/olpc/Documents/'+name+'-cover_page1.png')
        pag2.save('/home/olpc/Documents/'+name+'-cover_page2.png')


        print 'print_booklet not implemented'

    def page_format(pages, imposition):
        original_images = []
        quarters = []
        for page in pages:
            im = Image.open(page)
            original_images.append(im)
        for im in original_images:
            q = im.resize(qsize)
            quarters.append(q)
            print q.format, q.size, q.mode
        count = 0
        for quarter in quarters:
            count+= 1
            quarter.save('/home/olpc/Documents/'+name+'-'+str(count)+'.png')
        pg1 = Image.new('L',a4size)
        pg2 = Image.new('L',a4size)
        print pg1.format, pg1.size, pg1.mode
        print pg2.format, pg2.size, pg2.mode
        #paste quarters to pages
        qx = qsize[0]
        qy = qsize[1]
        pg1.paste(quarters[imposition[0]],(0,0))
        pg1.paste(quarters[imposition[1]],(qx,0))
        pg1.paste(quarters[imposition[2]],(0,qy))
        pg1.paste(quarters[imposition[3]],(qx,qy))
        pg2.paste(quarters[imposition[4]],(0,0))
        pg2.paste(quarters[imposition[5]],(qx,0))
        pg2.paste(quarters[imposition[6]],(0,qy))
        pg2.paste(quarters[imposition[7]],(qx,qy))
        print 'pg1', pg1.format, pg1.size, pg1.mode
        print 'pg2', pg2.format, pg2.size, pg2.mode
        return pg1, pg2
  
    def _upload_cb(self, entry):
        #at this point 'folder' contains the path 'namebase'
        #the path '/home/olpc/Documents/'+namebase 
        #points to a file or a folder (or is an error)
        #if a file, send it (with given extension)
        #if a file and no extension, assume .txt
        #if a folder, zip is as root.zip and send it
        #note if there is root.ext and a diretory root, root.ext gets sent
        self.title_set = False
        self.description_set = False
        self.collection_set = False
        #get name of folder
        folder = path(self.folder.get_text())
        print 'folder', folder
        if folder == 'untitled':
            self.label.set_text('Please enter the title of your document')
            return
        #clear any metadata files left from previous runs
        cmd = 'rm -rf ' + WORKPATH / folder+'.metadata'
        print cmd
        call(cmd, shell=True)
        #for now require it to be in Documents
        files = WORKPATH.files(folder+'*')
        dirs = WORKPATH.dirs()
        if files:
            fn = files[0].name
            if fn == files[0].namebase:
                fn  = fn + '.txt'
        elif dirs:
            #zip folder to ~/Documents as folder.zip
	    cmd = 'zip -rq '+folder+'.zip ' + folder
            print cmd
            cwd = WORKPATH
            print 'cwd', cwd
            call(cmd,cwd=cwd,shell=True)
            fn = folder + '.zip'   
        else:
            self.label.set_text('no file found to upload')
            return
        self.fn = path(fn).name
        self.label.set_text(self.fn + ' ready to upload')
       
        #get metadata including information for accession record and collectiom
        #ask user for title, description, collection

        label1 = Gtk.Label('title')
        self.entry1 = Gtk.Entry()
        self.entry1.connect('focus-out-event', self.title_set_cb)
        self.entry1.set_width_chars(50)
        self.fixed.put(label1, 50, 300)
        self.fixed.put(self.entry1, 200, 300)
        label2 = Gtk.Label('description')
        self.entry2 = Gtk.Entry()
        self.entry2.connect('focus-out-event', self.description_set_cb)
        self.entry2.set_width_chars(100)
        self.fixed.put(label2, 50, 400)
        self.fixed.put(self.entry2, 200,400)
        label3 = Gtk.Label('collection')
        self.entry3 = Gtk.Entry()
        self.entry3.connect('focus-out-event', self.collection_set_cb)
        self.entry3.set_width_chars(50)
        self.fixed.put(label3,50,500)
        self.fixed.put(self.entry3,200,500)
        button = Gtk.Button('Send')
        button.connect('clicked', self.send_cb)
        self.fixed.put(button,50, 600)
        self.fixed.show_all()

    def _inspect_cb(self, entry):
        #show metadata for specified jobject
        #use title in entry
        title = self.folder.get_text()
        #search datastore for title
        ds_objects, num_objects = datastore.find({'title_set_by_user': '%s' % '1'})
        print num_objects
        for obj in ds_objects:
            if obj.metadata['title'] == title:
                for key in obj.metadata.keys():
                    if not key in ['preview']:
                        print key, obj.metadata[key]
        
    def title_set_cb(self, widget, event):
        self._metadata['title'] = self.entry1.get_text()
        self.title_set = True

    def description_set_cb(self, widget, event):
        self._metadata['description'] = self.entry2.get_text()
        self.description_set = True

    def collection_set_cb(self, widget, event):
        self._metadata['collection'] = self.entry3.get_text()
        self.collection_set = True
  
    def send_cb(self, widget):
        if not self.title_set:
            self.label.set_text('Please supply a title')
            return
        if not self.description_set:
            self.label.set_text('Please supply a description')
            return
        if not self.collection_set:
            self.label.set_text('Please specify a collection')
            return

        ''' Returns nick from Sugar '''
        username = ''
        if using_gconf:
            client = gconf.client_get_default()
            username = client.get_string('/desktop/sugar/user/nick')
        else:
            username = profile.get_nick_name()

        serial_number = self._mfg_data('SN')
        print 'serial_number', serial_number

        #set metadata
        self._metadata['repository'] = 'xok'
        self._metadata['file'] = self.fn
        self._metadata['thumbnail'] = ''
        self._metadata['author'] = username        
        self._metadata['mime_type'] = 'application/zip'

        #now upload zipped file and metadata to schoolserver
        file_path = self.fn
        print 'file_path', file_path
        metadata_path = path(self.fn).namebase+'.metadata'
        print 'metadata_path', metadata_path
        cmd = 'rm -rf ' + metadata_path
        call(cmd, shell=True)
        try:

            fout = open(WORKPATH / metadata_path,'w')
            fout.write(str(self._metadata))
            fout.close()
        except:
            print 'exception writing metadata', sys.exc_info()[:2]
        test_files = path(WORKPATH).files(path(file_path).namebase+'*')
        print len(test_files)
        for file in test_files:
            print file 
        script = 'cd sandbox\nput '+ metadata_path+'\nput '+file_path+'\n'
        try:
            result,err = sftp(script,serial_number,WORKPATH)
        except:
            print 'sftp exception', sys.exc_info()[:2]
        if err:
            print 'sftp error', script, err
        if result:
            print 'sftp result', script, result
        #finally http://schoolserver/library/upload to inform Django of upload
        url = 'http://schoolserver/library/upload/'+serial_number+'/'+path(self.fn).namebase
        print 'url', url
        try:
            response =  urlopen(url)
        except:
            print 'exception', sys.exc_info()[:2]
            self.label.set_text('urlopen exception '+url)
        else:
            txt = response.read()
            self.label.set_text(txt)

    def get_sn(self):
        if have_ofw_tree():
            return read_ofw('mfg-data/SN')
        if sn:
            return sn
        sn = identifier_get_string('sn')
        if sn:
            return sn
        return 'SHF00000000'

    def _mfg_data(self, item):
        """Return mfg data item from mfg-data directory"""

        mfg_path = None
        for test_path in MFG_DATA_PATHS:
            if os.path.exists(test_path + item):
                mfg_path = test_path + item
                break
        if mfg_path == None:
            return ''

        v = self.__read_file(mfg_path)
        # Remove trailing 0 character, if any:
        if v != '' and ord(v[len(v)-1]) == 0:
            v = v[:len(v)-1]

        return v

    def laptop_serial_number(self):
        return self._mfg_data('SN')


    def __read_file(self, filename):
        """Read the entire contents of a file and return it as a string"""

        data = ''

        f = open(filename)
        try:
            data = f.read()
        finally:
            f.close()

       	return data
